@@ -2,9 +2,14 @@ from ._version import get_versions
 __version__ = get_versions()['version']
 del get_versions
 
-from andes import io  # NOQA
+from andes import io         # NOQA
+from andes import core       # NOQA
+from andes import models     # NOQA
+from andes import routines   # NOQA
+from andes import utils      # NOQA
+from andes import variables  # NOQA
+
 from andes.main import config_logger, run, load, prepare  # NOQA
-from andes import system  # NOQA
 from andes.system import System  # NOQA
 
 
