@@ -1,25 +1,29 @@
 import logging
-from typing import Optional, Union, Tuple
+from typing import Optional, Union, Tuple, List
 from andes.shared import np
+from andes.utils.func import interp_n2
 
 logger = logging.getLogger(__name__)
 
 
 class Discrete(object):
     """
-    Base class for discrete components which exports boolean flags.
+    Base discrete class.
+
+    Discrete classes export flag arrays (usually boolean) .
     """
 
-    def __init__(self, name=None, tex_name=None):
+    def __init__(self, name=None, tex_name=None, info=None):
         self.name = name
         self.tex_name = tex_name
+        self.info = info
         self.owner = None
         self.export_flags = []
         self.export_flags_tex = []
         self.x_set = list()
         self.y_set = list()  # NOT being used
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         """
         This function is called in ``l_update_var`` before evaluating equations.
 
@@ -70,22 +74,39 @@ class Discrete(object):
 
 class LessThan(Discrete):
     """
-    Less than (<) comparison function
+    Less than (<) comparison function.
+
+    Exports two flags: z1 and z0.
+    For elements satisfying the less-than condition, the corresponding z1 = 1.
+    z0 is the element-wise negation of z1.
+
+    Notes
+    -----
+    The default z0 and z1, if not enabled, can be set through the constructor.
     """
-    def __init__(self, u, bound, equal=False, enable=True, name=None, tex_name=None):
+    def __init__(self, u, bound, equal=False, enable=True, name=None, tex_name=None, cache=False,
+                 z0=0, z1=1):
         super().__init__(name=name, tex_name=tex_name)
         self.u = u
         self.bound = bound
-        self.equal = equal
-        self.enable = enable
-        self.z0 = 0  # if (u < bound) is False
-        self.z1 = 1  # if (u < bound) is True
+        self.equal: bool = equal
+        self.enable: bool = enable
+        self.cache: bool = cache
+        self._eval: bool = False  # if has been eval'ed and cached
+
+        self.z0 = np.array([z0])  # negation of `self.z1`
+        self.z1 = np.array([z1])  # if the less-than condition (u < bound) is True
         self.export_flags = ['z0', 'z1']
         self.export_flags_tex = ['z_0', 'z_1']
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
+        """
+        If enabled, set flags based on inputs. Use cached values if enabled.
+        """
         if not self.enable:
             return
+        if self.cache and self._eval:
+            return
 
         if not self.equal:
             self.z1[:] = np.less(self.u.v, self.bound.v)
@@ -94,12 +115,18 @@ class LessThan(Discrete):
 
         self.z0[:] = np.logical_not(self.z1)
 
+        self._eval = True
+
 
 class Limiter(Discrete):
     """
-    Base limiter class
+    Base limiter class.
 
-    This class compares values and sets limit values
+    This class compares values and sets limit values. Exported flags are `zi`, `zl` and `zu`.
+
+    Notes
+    -----
+    If not enabled, the default flags are ``zu = zl = 0``, ``zi = 1``.
 
     Parameters
     ----------
@@ -121,25 +148,23 @@ class Limiter(Discrete):
         Flags for violating the upper limit
     """
 
-    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None):
-        super().__init__(name=name, tex_name=tex_name)
+    def __init__(self, u, lower, upper, enable=True, name=None, tex_name=None, info=None):
+        super().__init__(name=name, tex_name=tex_name, info=info)
         self.u = u
         self.lower = lower
         self.upper = upper
         self.enable = enable
-        self.zu = 0
-        self.zl = 0
-        self.zi = 1
+
+        self.zu = np.array([0])
+        self.zl = np.array([0])
+        self.zi = np.array([1])
+
         self.export_flags = ['zl', 'zi', 'zu']
         self.export_flags_tex = ['z_l', 'z_i', 'z_u']
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         """
-        Evaluate `self.zu` and `self.zl`
-
-        Returns
-        -------
-
+        Evaluate the flags.
         """
         if not self.enable:
             return
@@ -150,7 +175,7 @@ class Limiter(Discrete):
 
 class SortedLimiter(Limiter):
     """
-    A comparer with the top value selection
+    A comparer with the top value selection.
 
     """
 
@@ -160,7 +185,7 @@ class SortedLimiter(Limiter):
         super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name)
         self.n_select = int(n_select) if n_select else 0
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         if not self.enable:
             return
         super().check_var()
@@ -184,26 +209,22 @@ class SortedLimiter(Limiter):
 
 class HardLimiter(Limiter):
     """
-    Hard limiter on an algebraic variable. This class is an alias of `Limiter`.
+    Hard limiter for algebraic or differential variable. This class is an alias of `Limiter`.
     """
     pass
 
 
-class WindupLimiter(Limiter):
-    def __init__(self, u, lower, upper,  enable=True, name=None, tex_name=None):
-        super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name)
-
-
-class AntiWindupLimiter(WindupLimiter):
+class AntiWindup(Limiter):
     """
     Anti-windup limiter.
 
-    The anti-windup limiter prevents the wind-up effect of a differential variable. The derivative of the
-    differential variable is reset if it continues to increase in the same direction after exceeding the limits.
+    Anti-windup limiter prevents the wind-up effect of a differential variable.
+    The derivative of the differential variable is reset if it continues to increase in the same direction
+    after exceeding the limits.
     During the derivative return, the limiter will be inactive ::
 
-      if x > xmax and x dot > 0: x = xmax and x dot = 0
-      if x < xmin and x dot < 0: x = xmin and x dot = 0
+        if x > xmax and x dot > 0: x = xmax and x dot = 0
+        if x < xmin and x dot < 0: x = xmin and x dot = 0
 
     This class takes one more optional parameter for specifying the equation.
 
@@ -218,7 +239,7 @@ class AntiWindupLimiter(WindupLimiter):
         super().__init__(u, lower, upper, enable=enable, name=name, tex_name=tex_name)
         self.state = state if state else u
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         """
         This function is empty. Defers `check_var` to `check_eq`.
         """
@@ -258,6 +279,11 @@ class Selector(Discrete):
 
     Names are in `s0`, `s1`, ... and `sn`
 
+    Warnings
+    --------
+    A potential bug when values in different inputs are equal.
+    FIXME: More than one flag will se true in this case.
+
     Examples
     --------
     Example 1: select the largest value between `v0` and `v1` and put it into vmax.
@@ -299,7 +325,7 @@ class Selector(Discrete):
         self.export_flags = [f's{i}' for i in range(len(self.input_vars))]
         self.export_flags_tex = [f's_{i}' for i in range(len(self.input_vars))]
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         """
         Set the i-th variable's flags to 1 if the return of the reduce function equals the i-th input.
         """
@@ -311,17 +337,50 @@ class Selector(Discrete):
 
 class Switcher(Discrete):
     """
-    Switcher class based on input parameters.
+    Switcher based on an input parameter.
 
     The switch class takes one v-provider, compares the input with each value in the option list, and exports
     one flag array for each option. The flags are 0-indexed.
+
+    Exported flags are named with `_s0`, `_s1`, ..., with a total number of `len(options)`. See the examples
+    section.
+
+    Notes
+    -----
+    Switches needs to be distinguished from Selector.
+
+    Switcher is for generating flags indicating option selection based on an input parameter. Selector is
+    for generating flags at run time based on variable values and a selection function.
+
+    Examples
+    --------
+    The IEEEST model takes an input for selecting the signal. Options are 1 through 6.
+    One can construct ::
+
+        self.IC = NumParam(info='input code 1-6')  # input code
+        self.SW = Switcher(u=self.IC, options=[1, 2, 3, 4, 5, 6])
+
+    If the IC values from the data file ends up being ::
+
+        self.IC.v = np.array([1, 2, 2, 4, 6])
+
+    Then, the exported flag arrays will be ::
+
+        {'IC_s0': np.array([1, 0, 0, 0, 0]),
+         'IC_s1': np.array([0, 1, 1, 0, 0]),
+         'IC_s2': np.array([0, 0, 0, 0, 0]),
+         'IC_s3': np.array([0, 0, 0, 1, 0]),
+         'IC_s4': np.array([0, 0, 0, 0, 0]),
+         'IC_s5': np.array([0, 0, 0, 0, 1])
+        }
     """
+
     def __init__(self, u, options: Union[list, Tuple], name: str = None, tex_name: str = None, cache=True):
         super().__init__(name=name, tex_name=tex_name)
         self.u = u
-        self.option: list = options
+        self.options: Union[List, Tuple] = options
         self.cache: bool = cache
-        self._eval = False  # if the flags has been evaluated
+        self._eval: bool = False  # if the flags has been evaluated
 
         for i in range(len(options)):
             self.__dict__[f's{i}'] = 0
@@ -329,23 +388,39 @@ class Switcher(Discrete):
         self.export_flags = [f's{i}' for i in range(len(options))]
         self.export_flags_tex = [f's_{i}' for i in range(len(options))]
 
-    def check_var(self):
-        """Set the switcher flags based on inputs. Uses cached flags if cache is set to True."""
+    def check_var(self, *args, **kwargs):
+        """
+        Set the switcher flags based on inputs. Uses cached flags if cache is set to True.
+
+        TODO: check if all inputs are valid
+        """
         if self.cache and self._eval:
             return
-        for i in range(len(self.option)):
-            self.__dict__[f's{i}'][:] = np.equal(self.u.v, self.option[i]).astype(np.float64)
+
+        for v in self.u.v:
+            if v not in self.options:
+                raise ValueError(f'option {v} is invalid for {self.owner.class_name}.{self.u.name}. '
+                                 f'Options are {self.options}.')
+        for i in range(len(self.options)):
+            self.__dict__[f's{i}'][:] = np.equal(self.u.v, self.options[i]).astype(np.float64)
 
         self._eval = True
 
+    def list2array(self, n):
+        """
+        This forces to evaluate Switcher upon System setup
+        """
+        super().list2array(n)
+        self.check_var()
+
 
 class DeadBand(Limiter):
-    """
-    Deadband with the direction of return.
+    r"""
+    Dead band with the direction of return.
 
     Parameters
     ----------
-    u
+    u : NumParam
         The pre-deadband input variable
     center : NumParam
         Neutral value of the output
@@ -407,17 +482,22 @@ class DeadBand(Limiter):
         """
         super().__init__(u, lower, upper, enable=enable)
         self.center = center
-        # default state if enable is False
-        self.zi = 0.
-        self.zl = 0.
-        self.zu = 0.
-        self.zur = 0.
-        self.zlr = 0.
+
+        # default state if not enabled
+        self.zi = np.array([0.])
+        self.zl = np.array([0.])
+        self.zu = np.array([0.])
+        self.zur = np.array([0.])
+        self.zlr = np.array([0.])
+
         self.export_flags = ['zl', 'zi', 'zu', 'zur', 'zlr']
         self.export_flags_tex = ['z_l', 'z_i', 'z_u', 'z_ur', 'z_lr']
 
-    def check_var(self):
+    def check_var(self, *args, **kwargs):
         """
+        Notes
+        -----
+
         Updates five flags: zi, zu, zl; zur, and zlr based on the following rules:
 
         zu:
@@ -425,6 +505,7 @@ class DeadBand(Limiter):
 
         zl:
           1 if u < lower; 0 otherwise.
+
         zi:
           not(zu or zl);
 
@@ -451,3 +532,128 @@ class DeadBand(Limiter):
         self.zu[:] = zu.astype(np.float64)
         self.zl[:] = zl.astype(np.float64)
         self.zi[:] = zi.astype(np.float64)
+
+
+class Delay(Discrete):
+    """
+    Delay class to memorize past variable values.
+
+    TODO: Add documentation.
+    """
+
+    def __init__(self, u, mode='step', delay=0, name=None, tex_name=None, info=None):
+        Discrete.__init__(self, name=name, tex_name=tex_name, info=info)
+
+        if mode not in ('step', 'time'):
+            raise ValueError(f'mode {mode} is invalid. Must be in "step" or "time"')
+
+        self.u = u
+        self.mode = mode
+        self.delay = delay
+        self.export_flags = ['v']
+        self.export_flags_tex = ['v']
+
+        self.t = np.array([0])
+        self.v = np.array([0])
+        self._v_mem = np.zeros((0, 1))
+        self.rewind = False
+
+    def list2array(self, n):
+        """
+        Allocate memory for storage arrays.
+        """
+        super().list2array(n)
+        if self.mode == 'step':
+            self._v_mem = np.zeros((n, self.delay + 1))
+            self.t = np.zeros(self.delay + 1)
+        else:
+            self._v_mem = np.zeros((n, 1))
+
+    def check_var(self, dae_t, *args, **kwargs):
+
+        # Storage:
+        # Output values is in the first col.
+        # Latest values are stored in /appended to the last column
+        self.rewind = False
+
+        if dae_t == 0:
+            self._v_mem[:] = self.u.v[:, None]
+
+        elif dae_t < self.t[-1]:
+            self.rewind = True
+            self.t[-1] = dae_t
+            self._v_mem[:, -1] = self.u.v
+
+        elif dae_t == self.t[-1]:
+            self._v_mem[:, -1] = self.u.v
+
+        elif dae_t > self.t[-1]:
+            if self.mode == 'step':
+                self.t[:-1] = self.t[1:]
+                self.t[-1] = dae_t
+
+                self._v_mem[:, :-1] = self._v_mem[:, 1:]
+                self._v_mem[:, -1] = self.u.v
+            else:
+                self.t = np.append(self.t, dae_t)
+                self._v_mem = np.hstack((self._v_mem, self.u.v[:, None]))
+
+                if dae_t - self.t[0] > self.delay:
+                    t_interp = dae_t - self.delay
+                    idx = np.argmax(self.t >= t_interp) - 1
+                    v_interp = interp_n2(t_interp,
+                                         self.t[idx:idx+2],
+                                         self._v_mem[:, idx:idx + 2])
+
+                    self.t[idx] = t_interp
+                    self._v_mem[:, idx] = v_interp
+
+                    self.t = np.delete(self.t, np.arange(0, idx))
+                    self._v_mem = np.delete(self._v_mem, np.arange(0, idx), axis=1)
+
+        self.v[:] = self._v_mem[:, 0]
+
+    def __repr__(self):
+        out = ''
+        out += f'v:\n {self.v}\n'
+        out += f't:\n {self.t}\n'
+        out += f'_v_men: \n {self._v_mem}\n'
+        return out
+
+
+class Average(Delay):
+    """
+    Compute the average of a BaseVar over a period of time or a number of samples.
+    """
+    def check_var(self, dae_t, *args, **kwargs):
+        Delay.check_var(self, dae_t, *args, **kwargs)
+
+        if dae_t == 0:
+            self.v[:] = self._v_mem[:, -1]
+            self._v_mem[:, :-1] = 0
+            return
+        else:
+            nt = len(self.t)
+            self.v[:] = 0.5 * np.sum((self._v_mem[:, 1-nt:] + self._v_mem[:, -nt:-1]) *
+                                     (self.t[1:] - self.t[:-1]), axis=1) / (self.t[-1] - self.t[0])
+
+
+class Derivative(Delay):
+    """
+    Compute the derivative of an algebraic variable using numerical differentiation.
+    """
+    def __init__(self, u, name=None, tex_name=None, info=None):
+        Delay.__init__(self, u=u, mode='step', delay=1,
+                       name=name, tex_name=tex_name, info=info)
+
+    def check_var(self, dae_t, *args, **kwargs):
+        Delay.check_var(self, dae_t, *args, **kwargs)
+
+        # Note:
+        #    Very small derivatives (< 1e-8) could cause numerical problems (chattering).
+        #    Need to reset the output to zero following a rewind.
+        if (dae_t == 0) or (self.rewind is True):
+            self.v[:] = 0
+        else:
+            self.v[:] = (self._v_mem[:, 1] - self._v_mem[:, 0]) / (self.t[1] - self.t[0])
+            self.v[np.where(self.v < 1e-8)] = 0
