@@ -14,6 +14,7 @@ logger = logging.getLogger(__name__)
 
 input_formats = {
     'xlsx': ('xlsx',),
+    'json': ('json',),
     'matpower': ('m', ),
     'psse': ('raw', 'dyr'),
 }
@@ -21,7 +22,21 @@ input_formats = {
 # Output formats is a dictionary of supported output formats and their extensions
 # The static data will be written by write() function and the addfile by writeadd()
 
-output_formats = ['']
+output_formats = {
+    'xlsx': ('xlsx',),
+    'json': ('json',),
+}
+
+
+def get_output_ext(out_format):
+    if (out_format is None) or (out_format is True):
+        logger.warning('Dump to XLSX format by default')
+        return 'xlsx'
+    if out_format in output_formats:
+        return output_formats[out_format][0]
+    else:
+        logger.error(f"Dump format <{out_format}> not supported.")
+        return ''
 
 
 def guess(system):
@@ -94,44 +109,34 @@ def parse(system):
             logger.error('Input format is not specified and cannot be inferred.')
             return False
 
-    input_format = system.files.input_format
-    add_format = system.files.add_format
-
-    # exit if the format parser could not be imported
-    try:
-        parser = importlib.import_module('.' + input_format, __name__)
-        if add_format:
-            addparser = importlib.import_module('.' + add_format, __name__)
-    except ImportError:
-        logger.error(f'Parser for {input_format} format not found. Program will exit.')
-        return False
-
     # try parsing the base case file
-    logger.info('Parsing input file <{:s}>'.format(system.files.case))
-
+    logger.info(f'Parsing input file "{system.files.case}"')
+    input_format = system.files.input_format
+    parser = importlib.import_module('.' + input_format, __name__)
     if not parser.read(system, system.files.case):
-        logger.error(
-            'Error parsing case file {:s} with {:s} format parser.'.format(
-                system.files.fullname, input_format))
+        logger.error(f'Error parsing case file {system.files.fullname} with {input_format} format parser.')
         return False
 
-    # Try parsing the addfile
-    if system.files.addfile:
-        if not system.files.add_format:
-            logger.error('Unknown addfile format.')
-            return
-        logger.info(f'Parsing additional file {system.files.addfile}')
-        if not addparser.read_add(system, system.files.addfile):
-            logger.error('Error parsing addfile {:s} with {:s} parser.'.format(system.files.addfile, input_format))
-            return False
-
     _, s = elapsed(t)
-    logger.info(f'Input file {system.files.fullname} parsed in {s}.')
+    logger.info(f'Input file parsed in {s}.')
+
+    # Try parsing the addfile
+    t, _ = elapsed()
+
+    if system.files.addfile:
+        logger.info(f'Parsing additional file "{system.files.addfile}"')
+        add_format = system.files.add_format
+        add_parser = importlib.import_module('.' + add_format, __name__)
+        if not add_parser.read_add(system, system.files.addfile):
+            logger.error(f'Error parsing addfile {system.files.addfile} with {input_format} parser.')
+            return False
+        _, s = elapsed(t)
+        logger.info(f'Addfile parsed in {s}.')
 
     return True
 
 
-def dump(system, output_format):
+def dump(system, output_format, full_path=None, overwrite=False):
     """
     Dump the System data into the requested output format.
 
@@ -150,17 +155,26 @@ def dump(system, output_format):
     if system.files.no_output:
         return
 
-    if not isinstance(output_format, str):
+    if (output_format is None) or (output_format is True):
         output_format = 'xlsx'
 
-    outfile = system.files.dump
+    output_ext = get_output_ext(output_format)
+    if output_ext == '':
+        return
+
+    if full_path is not None:
+        system.files.dump = full_path
+    else:
+        system.files.dump = os.path.join(system.files.output_path,
+                                         system.files.name + '.' + output_ext)
+
     writer = importlib.import_module('.' + output_format, __name__)
 
     t, _ = elapsed()
-    ret = writer.write(system, outfile)
+    ret = writer.write(system, system.files.dump, overwrite=overwrite)
     _, s = elapsed(t)
     if ret:
-        logger.info(f'Converted file {system.files.dump} written in {s}.')
+        logger.info(f'Format conversion completed in {s}.')
         return True
     else:
         logger.error('Format conversion failed.')
