@@ -1,4 +1,5 @@
 import logging
+from andes.utils.func import list_flatten
 from andes.shared import np
 
 logger = logging.getLogger(__name__)
@@ -54,14 +55,24 @@ class GroupBase(object):
 
     def idx2model(self, idx):
         """
-        Find model name for the given idx
+        Find model name for the given idx.
         """
         ret = []
+        single = False
+        if not isinstance(idx, (list, tuple, np.ndarray)):
+            single = True
+            idx = (idx, )
+        elif len(idx) > 0 and isinstance(idx[0], (list, tuple, np.ndarray)):
+            idx = list_flatten(idx)
+
         for i in idx:
             try:
                 ret.append(self._idx2model[i])
             except KeyError:
                 raise KeyError(f'Group <{self.class_name}> does not contain idx <{i}>')
+
+        if single:
+            ret = ret[0]
         return ret
 
     def get(self, src: str, idx, attr: str = 'v'):
@@ -128,7 +139,7 @@ class GroupBase(object):
         for idx, idx_found in enumerate(zip(*indices_found)):
             if idx_found.count(False) == len(idx_found):
                 missing_values = [item[idx] for item in values]
-                raise IndexError(f'{keys} = {missing_values} not found in {self.__class__.__name__}')
+                raise IndexError(f'{keys} = {missing_values} not found in {self.class_name}')
             for item in idx_found:
                 if item is not False:
                     out.append(item)
@@ -141,7 +152,7 @@ class GroupBase(object):
 
     def _check_idx(self, idx):
         if idx is None:
-            raise IndexError(f'{self.__class__.__name__}: idx cannot be None')
+            raise IndexError(f'{self.class_name}: idx cannot be None')
 
     def get_next_idx(self, idx=None, model_name=None):
         """
@@ -167,15 +178,19 @@ class GroupBase(object):
                 # name is good
                 pass
             else:
-                logger.debug(f"{self.class_name}: conflict idx {idx}. Data may be inconsistent.")
+                logger.warning(f"Group <{self.class_name}>: idx={idx} is used by "
+                               f"{self.idx2model(idx).class_name}. "
+                               f"Data may be inconsistent.")
                 need_new = True
         else:
             need_new = True
 
         if need_new is True:
-            count = len(self._idx2model)
+            count = self.n
             while True:
-                idx = model_name + '_' + str(count)
+                # IMPORTANT: automatically assigned index is 1-indexed. Namely, `GENCLS_1` is the first generator.
+                # This is because when we say, for example, `GENCLS_10`, people usually assume it starts at 1.
+                idx = model_name + '_' + str(count + 1)
                 if idx not in self._idx2model:
                     break
                 else:
@@ -248,6 +263,11 @@ class Collection(GroupBase):
     pass
 
 
+class Calculation(GroupBase):
+    """Group of classes that calculates based on other models."""
+    pass
+
+
 class StaticGen(GroupBase):
     """
     Static generator group for power flow calculation
@@ -282,8 +302,8 @@ class SynGen(GroupBase):
     """
     def __init__(self):
         super().__init__()
-        self.common_params.extend(('Sn', 'Vn', 'fn', 'bus'))
-        self.common_vars.extend(('omega', 'delta', 'tm', 'vf'))
+        self.common_params.extend(('Sn', 'Vn', 'fn', 'bus', 'M', 'D'))
+        self.common_vars.extend(('omega', 'delta', 'tm', 'te', 'vf'))
 
 
 class TurbineGov(GroupBase):
@@ -301,7 +321,15 @@ class Exciter(GroupBase):
     """
     def __init__(self):
         super().__init__()
-        self.common_vars.extend(('vout', ))
+        self.common_params.extend(('syn', ))
+        self.common_vars.extend(('vout', 'vi', ))
+
+
+class PSS(GroupBase):
+    """Power system stabilizer group."""
+    def __init__(self):
+        super().__init__()
+        self.common_vars.extend(('vsout', ))
 
 
 class Experimental(GroupBase):
@@ -326,4 +354,6 @@ class TimedEvent(GroupBase):
 
 class FreqMeasurement(GroupBase):
     """Frequency measurements."""
-    pass
+    def __init__(self):
+        super().__init__()
+        self.common_vars.extend(('f', ))
