@@ -35,7 +35,8 @@ class Toggler(TogglerData, Model):
                 u0 = instance.get(src='u', attr='v', idx=self.dev.v[i])
                 instance.set(src='u', attr='v', idx=self.dev.v[i], value=1-u0)
                 action = True
-                tqdm.write(f'<Toggle {i}>: Status of {self.model.v[i]}.{self.dev.v[i]} changed to {1-u0}.')
+                tqdm.write(f'<Toggle {self.idx.v[i]}>: '
+                           f'Status of {self.model.v[i]}.{self.dev.v[i]} changed to {1-u0} at t={self.t.v[i]}sec.')
         return action
 
 
@@ -58,12 +59,12 @@ class Fault(ModelData, Model):
                              unit='second',
                              callback=self.clear_fault,
                              )
-        self.xf = NumParam(info='Fault to ground impedance',
+        self.xf = NumParam(info='Fault to ground impedance (positive)',
                            unit='p.u.(sys)',
                            default=1e-4,
                            tex_name='x_f',
                            )
-        self.rf = NumParam(info='Fault to ground resistance',
+        self.rf = NumParam(info='Fault to ground resistance (positive)',
                            unit='p.u.(sys)',
                            default=0,
                            tex_name='x_f',
@@ -79,10 +80,8 @@ class Fault(ModelData, Model):
                                v_str='im(1/(rf + 1j * xf))',
                                )
 
-        # uf: an internal flag of whether the fault is in action (1) or not
-        self.uf = ConstService(tex_name='u_f',
-                               v_str='0',
-                               )
+        # uf: an internal flag of whether the fault is in action (1) or not (0)
+        self.uf = ConstService(tex_name='u_f', v_str='0')
 
         self.a = ExtAlgeb(model='Bus',
                           src='a',
@@ -104,24 +103,28 @@ class Fault(ModelData, Model):
 
     def apply_fault(self, is_time: np.ndarray):
         """
-        Apply fault and store pre-fault bus voltages to ``self._vstore``.
+        Apply fault and store pre-fault algebraic variables (voltages and other algebs) to `self._vstore`.
         """
+        action = False
         for i in range(self.n):
             if is_time[i] and (self.u.v[i] == 1):
                 self.uf.v[i] = 1
-                self._vstore = np.array(self.system.Bus.v.v)
-                tqdm.write(f'<Fault {i}>: Applying fault on {self.bus.v[i]} at {self.tf.v[i]}.')
-                return True
-        return False
+                self._vstore = np.array(self.system.dae.y[self.system.Bus.n:])
+                tqdm.write(f'<Fault {self.idx.v[i]}>: '
+                           f'Applying fault on Bus (idx={self.bus.v[i]}) at t={self.tf.v[i]}sec.')
+                action = True
+        return action
 
     def clear_fault(self, is_time: np.ndarray):
         """
-        Clear fault and restore pre-fault bus voltages.
+        Clear fault and restore pre-fault bus algebraic variables (voltages and others).
         """
+        action = False
         for i in range(self.n):
             if is_time[i] and (self.u.v[i] == 1):
                 self.uf.v[i] = 0
-                self.system.dae.y[self.system.Bus.v.a] = self._vstore
-                tqdm.write(f'<Fault {i}>: Clearing fault on {self.bus.v[i]} at {self.tc.v[i]}.')
-                return True
-        return False
+                self.system.dae.y[self.system.Bus.n:] = self._vstore
+                tqdm.write(f'<Fault {self.idx.v[i]}>: '
+                           f'Clearing fault on Bus (idx={self.bus.v[i]}) at t={self.tc.v[i]}sec.')
+                action = True
+        return action
