@@ -10,13 +10,31 @@ logger = logging.getLogger(__name__)
 
 
 class PFlow(BaseRoutine):
-
+    """
+    Power flow calculation routine.
+    """
     def __init__(self, system=None, config=None):
         super().__init__(system, config)
         self.config.add(OrderedDict((('tol', 1e-6),
                                      ('max_iter', 25),
+                                     ('method', 'NR'),
+                                     ('n_factorize', 4),
                                      ('report', 1),
                                      )))
+        self.config.add_extra("_help",
+                              tol="convergence tolerance",
+                              max_iter="max. number of iterations",
+                              method="calculation method",
+                              n_factorize="first N iterations to factorize Jacobian in dishonest method",
+                              report="write output report",
+                              )
+        self.config.add_extra("_alt",
+                              tol="float",
+                              method=("NR", "dishonest"),
+                              max_iter=">=10",
+                              n_factorize=">0",
+                              report=(0, 1),
+                              )
         self.models = system.find_models('pflow')
 
         self.converged = False
@@ -25,22 +43,24 @@ class PFlow(BaseRoutine):
         self.niter = None
         self.mis = []
 
-    def _initialize(self):
+    def init(self):
         self.converged = False
         self.inc = None
         self.A = None
         self.niter = None
         self.mis = []
-        self.system.initialize(self.models)
+        self.system.init(self.models)
         logger.info('Power flow initialized.')
         return self.system.dae.xy
 
     def nr_step(self):
         """
-        Single stepping for Newton Raphson method
+        Single step using Newton-Raphson method.
+
         Returns
         -------
-
+        float
+            maximum absolute mismatch
         """
         system = self.system
         # evaluate discrete, differential, algebraic, and jacobians
@@ -51,7 +71,12 @@ class PFlow(BaseRoutine):
         system.l_check_eq()
         system.l_set_eq()
         system.fg_to_dae()
-        system.j_update()
+
+        if self.config.method == 'NR':
+            system.j_update()
+        elif self.config.method == 'dishonest':
+            if self.niter < self.config.n_factorize:
+                system.j_update()
 
         # prepare and solve linear equations
         self.inc = -matrix([matrix(system.dae.f),
@@ -73,24 +98,28 @@ class PFlow(BaseRoutine):
         return mis
 
     def summary(self):
+        """
+        Output a summary for the PFlow routine.
+        """
         out = list()
         out.append('')
         out.append('-> Power flow calculation')
-        out.append('Method: Full Newton-Raphson method')
+        out.append(f'Method: {self.config.method} method')
         out_str = '\n'.join(out)
         logger.info(out_str)
 
     def run(self, **kwargs):
         """
-        Full Newton-Raphson method
+        Full Newton-Raphson method.
 
         Returns
         -------
-
+        bool
+            convergence status
         """
         system = self.system
         self.summary()
-        self._initialize()
+        self.init()
         if system.dae.m == 0:
             logger.error("Loaded case contains no power flow element.")
             return False
@@ -129,11 +158,14 @@ class PFlow(BaseRoutine):
         else:
             logger.info(f'Converged in {self.niter+1} iterations in {s1}.')
             if self.config.report:
-                system.PFlow.write_report()
+                system.PFlow.report()
 
         return self.converged
 
-    def write_report(self):
+    def report(self):
+        """
+        Write power flow report to text file.
+        """
         if self.system.files.no_output is False:
             r = Report(self.system)
             r.write()
@@ -181,12 +213,12 @@ class PFlow(BaseRoutine):
 
         """
         system = self.system
-        system.initialize()
+        system.init()
         v0 = system.dae.xy
         try:
             ret = newton_krylov(self._fg_wrapper, v0, verbose=verbose)
         except ValueError as e:
-            logger.error('Mismatch is not correctable. Equations may be intrinsically unsolvable.')
+            logger.error('Mismatch is not correctable. Equations may be unsolvable.')
             raise e
 
         return ret
