@@ -195,9 +195,9 @@ def get_log_dir():
 def confirm_overwrite(outfile, overwrite=None):
     if os.path.isfile(outfile):
         if overwrite is None:
-            choice = input(f'File <{outfile}> already exist. Overwrite? [y/N]').lower()
+            choice = input(f'File "{outfile}" already exist. Overwrite? [y/N]').lower()
             if len(choice) == 0 or choice[0] != 'y':
-                logger.warning(f'File <{outfile}> not overwritten.')
+                logger.warning(f'File "{outfile}" not overwritten.')
                 return False
         elif overwrite is False:
             return False
